@@ -23,26 +23,26 @@
 
 #include "animated.h"
 
-#include <set>
-
 class AnimatedCollection {
  public:
   AnimatedCollection();
   ~AnimatedCollection();
-  void insert(Animated*);
-  void remove(Animated*);
+  void add(Animated*);
+  void reserve(int N);
+  void clear();
 
   void recalculateBboxMap();
-  std::set<Animated*> bboxOverlapsWith(const Animated*) const;
-
-  const std::set<Animated*>& asSet() const;
+  size_t bboxOverlapsWith(const Animated*, Animated*** ret) const;
+  size_t bboxOverlapsWith(const double lower[3], const double upper[3], Animated*** ret) const;
 
  private:
-  std::set<Animated*> store;
   void* map;
   void* alloc;
   void** retlist;
   int ntot;
+  int nreserved;
+  int* rect_indices;
+  void* input;
 };
 
 #endif /* ANIMATED_COLLECTION_H */
