@@ -46,8 +46,8 @@ Ball::Ball(Game &game, int role) : Animated(game, role, 7) {
 
   dizzy_r = dizzy_p = 0.0;
 
-  primaryColor = Color(0.8, 0.8, 0.8, 1.);
-  secondaryColor = Color(0.1, 0.1, 0.1, 1.);
+  primaryColor = Color(SRGBColor(0.8, 0.8, 0.8, 1.));
+  secondaryColor = Color(SRGBColor(0.1, 0.1, 0.1, 1.));
 
   gravity = 8.0;
   bounceFactor = 0.8;
@@ -73,20 +73,26 @@ Ball::Ball(Game &game, int role) : Animated(game, role, 7) {
   reflectivity = 0.0;
   metallic = 0;
   dontReflectSelf = 0;
+
+  bufferSpike = false;
+  bufferFloat = false;
+  bufferExtraLife = false;
+  bufferDizzy = false;
+  bufferJump = false;
 }
 Ball::~Ball() {
   setReflectivity(0.0, 0);  // This effectivly deallocates all environment maps */
 }
 
-void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
-                           const GLuint *vaolist, bool /*mustUpdate*/) const {
-  if (!is_on) return;
+void Ball::updateBuffers(const GLuint *idxbufs, const GLuint *databufs, const GLuint *vaolist,
+                         bool firstCall) {
+  if (!is_on && !firstCall) return;
 
   Color color = primaryColor.toOpaque();
   if (0) {
     /* useful debug code */
     if (inTheAir) {
-      for (int i = 0; i < 3; i++) { color.v[i] = 65535 - color.v[i]; }
+      for (int i = 0; i < 3; i++) { color.v[i] = 1.f - color.v[i]; }
     }
   }
   GLfloat loc[3] = {(GLfloat)position[0], (GLfloat)position[1], (GLfloat)(position[2] - sink)};
@@ -96,48 +102,40 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
       phase = std::min(modTimeLeft[MOD_GLASS] / 2.0, phase);
     else
       phase = 1.0;
-    color = Color::mix(phase, color, Color(0.8, 0.8, 0.8, 0.5));
+    color = Color::mix(phase, color, Color(SRGBColor(0.8, 0.8, 0.8, 0.5)));
   } else if (modTimeLeft[MOD_FROZEN]) {
     double phase = std::min(modTimePhaseIn[MOD_FROZEN] / 2.0, 1.0);
     if (modTimeLeft[MOD_FROZEN] > 0)
       phase = std::min(modTimeLeft[MOD_FROZEN] / 2.0, phase);
     else
       phase = 1.0;
-    color = Color::mix(phase, color, Color(0.4, 0.4, 0.9, 0.6));
+    color = Color::mix(phase, color, Color(SRGBColor(0.4, 0.4, 0.9, 0.6)));
   }
 
-  {
+  if (firstCall || bufferBallColor != color) {
+    bufferBallColor = color;
+
     /* Construct VBO for main ball */
     int ntries = 0;
     int nverts = 0;
     int detail;
     switch (ballResolution) {
     case BALL_HIRES:
-      detail = 20;
+      detail = 3;
       break;
     default:
     case BALL_NORMAL:
-      detail = 14;
+      detail = 2;
       break;
     case BALL_LORES:
-      detail = 8;
+      detail = 2;
       break;
     }
     countObjectSpherePoints(&ntries, &nverts, detail);
     GLfloat *data = new GLfloat[nverts * 8];
     ushort *idxs = new ushort[ntries * 3];
-    Matrix3d rotation;
-    if (modTimeLeft[MOD_EXTRA_LIFE]) {
-      Matrix4d ref;
-      identityMatrix(ref);
-      rotateY(M_PI / 3, ref);
-      for (int i = 0; i < 3; i++)
-        for (int j = 0; j < 3; j++) rotation[i][j] = ref[i][j];
-    } else {
-      for (int i = 0; i < 3; i++)
-        for (int j = 0; j < 3; j++) rotation[j][i] = rotations[i][j];
-    }
-    placeObjectSphere(data, idxs, 0, loc, rotation, radius, detail, color);
+
+    placeObjectSphere(data, idxs, 0, detail, color);
 
     glBindVertexArray(vaolist[0]);
     glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
@@ -149,7 +147,10 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
     delete[] idxs;
   }
 
-  if (modTimeLeft[MOD_SPIKE]) {
+  if (modTimeLeft[MOD_SPIKE] && (!bufferSpike || secondaryColor != bufferSpikeColor)) {
+    bufferSpikeColor = secondaryColor;
+    bufferSpike = true;
+
     /* spikes correspond to icosahedral faces */
     const GLfloat w = (1 + std::sqrt(5)) / 2;
     double icoverts[12][3] = {
@@ -162,18 +163,8 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
         {3, 8, 9},   {4, 9, 5},  {2, 4, 11}, {6, 2, 10}, {8, 6, 7},   {9, 8, 1},
     };
 
-    GLfloat phase = std::min(modTimePhaseIn[MOD_SPIKE] / 2.0f, 1.0f);
-    if (modTimeLeft[MOD_SPIKE] > 0)
-      phase = std::min(modTimeLeft[MOD_SPIKE] / 2.0f, phase);
-    else
-      phase = 1.0;
-    GLfloat scale = radius * (0.5 + 0.5 * phase);
-
     Color sco = secondaryColor.toOpaque();
     GLfloat flat[3] = {0.f, 0.f, 0.f};
-    Matrix3d trot;
-    for (int i = 0; i < 3; i++)
-      for (int j = 0; j < 3; j++) { trot[i][j] = rotations[j][i]; }
 
     GLfloat data[20 * 4][8];
     ushort idxs[20 * 3][3];
@@ -184,16 +175,12 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
       centroid = centroid / 3.0;
 
       Coord3d spike = centroid;
-      spike = spike * 0.87 * scale;
-      Coord3d sub = useMatrix(trot, spike);
-      pos += packObjectVertex(pos, loc[0] + sub[0], loc[1] + sub[1], loc[2] + sub[2], 0., 0.,
-                              sco, flat);
+      spike = spike * 0.87;
+      pos += packObjectVertex(pos, spike[0], spike[1], spike[2], 0., 0., sco, flat);
       for (int j = 2; j >= 0; j--) {
         Coord3d edge(icoverts[icofaces[i][j]]);
-        edge = 0.3 * scale * (0.1 * centroid + 0.9 * edge);
-        sub = useMatrix(trot, edge);
-        pos += packObjectVertex(pos, loc[0] + sub[0], loc[1] + sub[1], loc[2] + sub[2], 0., 0.,
-                                sco, flat);
+        edge = 0.3 * (0.1 * centroid + 0.9 * edge);
+        pos += packObjectVertex(pos, edge[0], edge[1], edge[2], 0., 0., sco, flat);
       }
     }
 
@@ -257,9 +244,11 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
   }
 
   // Handle modifiers
-  if (modTimeLeft[MOD_FLOAT]) {
-    Color stripeA(1., 1., 1., 1.);
-    Color stripeB(1., 0.2, 0.2, 1.);
+  if (modTimeLeft[MOD_FLOAT] && !bufferFloat) {
+    bufferFloat = true;
+
+    Color stripeA(SRGBColor(1., 1., 1., 1.));
+    Color stripeB(SRGBColor(1., 0.2, 0.2, 1.));
 
     GLfloat data[60][8];
     ushort idxs[40][3];
@@ -268,7 +257,7 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
       int i0 = i, i1 = (i + 1) % 10;
       const Color &ringcolor = i % 2 ? stripeA : stripeB;
 
-      GLfloat ir = radius * 1.0f, mr = radius * 1.2f, lr = radius * 1.4f, h = radius * 0.2f;
+      GLfloat ir = 1.0f, mr = 1.2f, lr = 1.4f, h = 0.2f;
 
       GLfloat inormal0[3] = {(GLfloat)-sin10[i0], (GLfloat)-cos10[i0], 0.f};
       GLfloat inormal1[3] = {(GLfloat)-sin10[i1], (GLfloat)-cos10[i1], 0.f};
@@ -276,20 +265,20 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
       GLfloat onormal1[3] = {(GLfloat)sin10[i1], (GLfloat)cos10[i1], 0.f};
       GLfloat unormal[3] = {0.f, 0.f, 1.f};
 
-      pos += packObjectVertex(pos, loc[0] + sin10[i0] * ir, loc[1] + cos10[i0] * ir, loc[2],
-                              0., 0., ringcolor, inormal0);
-      pos += packObjectVertex(pos, loc[0] + sin10[i1] * ir, loc[1] + cos10[i1] * ir, loc[2],
-                              0., 0., ringcolor, inormal1);
+      pos += packObjectVertex(pos, sin10[i0] * ir, cos10[i0] * ir, 0., 0., 0., ringcolor,
+                              inormal0);
+      pos += packObjectVertex(pos, sin10[i1] * ir, cos10[i1] * ir, 0., 0., 0., ringcolor,
+                              inormal1);
 
-      pos += packObjectVertex(pos, loc[0] + sin10[i0] * mr, loc[1] + cos10[i0] * mr,
-                              loc[2] + h, 0., 0., ringcolor, unormal);
-      pos += packObjectVertex(pos, loc[0] + sin10[i1] * mr, loc[1] + cos10[i1] * mr,
-                              loc[2] + h, 0., 0., ringcolor, unormal);
+      pos +=
+          packObjectVertex(pos, sin10[i0] * mr, cos10[i0] * mr, h, 0., 0., ringcolor, unormal);
+      pos +=
+          packObjectVertex(pos, sin10[i1] * mr, cos10[i1] * mr, h, 0., 0., ringcolor, unormal);
 
-      pos += packObjectVertex(pos, loc[0] + sin10[i0] * lr, loc[1] + cos10[i0] * lr, loc[2],
-                              0., 0., ringcolor, onormal0);
-      pos += packObjectVertex(pos, loc[0] + sin10[i1] * lr, loc[1] + cos10[i1] * lr, loc[2],
-                              0., 0., ringcolor, onormal1);
+      pos += packObjectVertex(pos, sin10[i0] * lr, cos10[i0] * lr, 0., 0., 0., ringcolor,
+                              onormal0);
+      pos += packObjectVertex(pos, sin10[i1] * lr, cos10[i1] * lr, 0., 0., 0., ringcolor,
+                              onormal1);
 
       ushort pts[4][3] = {{1, 0, 3}, {0, 2, 3}, {2, 5, 3}, {2, 4, 5}};
       for (int k = 0; k < 12; k++) { idxs[4 * i + k / 3][k % 3] = pts[k / 3][k % 3] + 6 * i; }
@@ -302,17 +291,20 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
     configureObjectAttributes();
   }
 
-  if (modTimeLeft[MOD_EXTRA_LIFE]) {
+  if (modTimeLeft[MOD_EXTRA_LIFE] &&
+      (!bufferExtraLife || bufferExtraLifeColor != primaryColor)) {
+    bufferExtraLife = true;
+    bufferExtraLifeColor = primaryColor;
+
     int ntries = 0;
     int nverts = 0;
-    int detail = 5;
+    int detail = 3;
     countObjectSpherePoints(&ntries, &nverts, detail);
     GLfloat *data = new GLfloat[nverts * 8];
     ushort *idxs = new ushort[ntries * 3];
     Color color = primaryColor;
-    color.v[3] = 32768;
-    Matrix3d identity = {{1.f, 0.f, 0.f}, {0.f, 1.f, 0.f}, {0.f, 0.f, 1.f}};
-    placeObjectSphere(data, idxs, 0, loc, identity, radius * 1.25, detail, color);
+    color.v[3] = 0.5f;
+    placeObjectSphere(data, idxs, 0, detail, color);
 
     glBindVertexArray(vaolist[4]);
     glBindBuffer(GL_ARRAY_BUFFER, databufs[4]);
@@ -324,27 +316,22 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
     delete[] idxs;
   }
 
-  if (modTimeLeft[MOD_JUMP]) {
+  if (modTimeLeft[MOD_JUMP] && !bufferJump) {
+    bufferJump = true;
+
     GLfloat data[9][8];
     ushort idxs[3][3];
     char *pos = (char *)data;
-    GLfloat z = 1.1 + 1.0 * std::fmod(game.gameTime, 1.0);
-    GLfloat frad = (GLfloat)radius;
+    GLfloat z = 1.1;
     GLfloat corners[9][2] = {
-        {frad * -.3f, frad * z},         {frad * .3f, frad * (z + .9f)},
-        {frad * .3f, frad * z},          {frad * .3f, frad * (z + .9f)},
-        {frad * -.3f, frad * z},         {frad * -.3f, frad * (z + .9f)},
-        {frad * -.6f, frad * (z + .9f)}, {0.f, frad * (z + 2.1f)},
-        {frad * .6f, frad * (z + .9f)},
+        {-.3f, z},         {.3f, (z + .9f)},  {.3f, z},          {.3f, (z + .9f)}, {-.3f, z},
+        {-.3f, (z + .9f)}, {-.6f, (z + .9f)}, {0.f, (z + 2.1f)}, {.6f, (z + .9f)},
     };
 
-    Color color(1., 1., 1., 0.5);
+    Color color(SRGBColor(1., 1., 1., 0.5));
     GLfloat flat[3] = {0.f, 0.f, 0.f};
-    GLfloat spin = game.gameTime * 0.15;
     for (int i = 0; i < 9; i++) {
-      pos += packObjectVertex(pos, loc[0] + std::cos(spin) * corners[i][0],
-                              loc[1] + std::sin(spin) * corners[i][0], loc[2] + corners[i][1],
-                              0., 0., color, flat);
+      pos += packObjectVertex(pos, corners[i][0], 0., corners[i][1], 0., 0., color, flat);
       idxs[i / 3][i % 3] = i;
     }
 
@@ -356,24 +343,25 @@ void Ball::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
     configureObjectAttributes();
   }
 
-  if (modTimeLeft[MOD_DIZZY]) {
+  if (modTimeLeft[MOD_DIZZY] && !bufferDizzy) {
+    bufferDizzy = true;
+
     GLfloat data[12][8];
     char *pos = (char *)data;
-    Color color(1., 1., 1., 0.5);
+    Color color(SRGBColor(1., 1., 1., 0.5));
     GLfloat flat[3] = {0.f, 0.f, 0.f};
     for (int i = 0; i < 3; i++) {
-      GLfloat angle = 0.5 * M_PI * game.gameTime + 2 * i * M_PI / 3;
-      GLfloat frad = radius;
-      GLfloat corners[4][3] = {{-0.6f * frad, 1.5f * frad, -0.1f * frad},
-                               {+0.6f * frad, 1.5f * frad, -0.1f * frad},
-                               {+0.6f * frad, 1.5f * frad, 1.1f * frad},
-                               {-0.6f * frad, 1.5f * frad, 1.1f * frad}};
+      GLfloat angle = 2 * i * M_PI / 3;
+      GLfloat corners[4][3] = {{-0.6f, 1.5f, -0.1f},
+                               {+0.6f, 1.5f, -0.1f},
+                               {+0.6f, 1.5f, 1.1f},
+                               {-0.6f, 1.5f, 1.1f}};
       GLfloat txco[4][2] = {{0.f, 1.f}, {1.f, 1.f}, {1.f, 0.f}, {0.f, 0.f}};
       for (int k = 0; k < 4; k++) {
         pos += packObjectVertex(
-            pos, loc[0] - std::sin(angle) * corners[k][0] + std::cos(angle) * corners[k][1],
-            loc[1] + std::cos(angle) * corners[k][0] + std::sin(angle) * corners[k][1],
-            loc[2] + corners[k][2], txco[k][0], txco[k][1], color, flat);
+            pos, -std::sin(angle) * corners[k][0] + std::cos(angle) * corners[k][1],
+            +std::cos(angle) * corners[k][0] + std::sin(angle) * corners[k][1], +corners[k][2],
+            txco[k][0], txco[k][1], color, flat);
       }
     }
 
@@ -402,7 +390,7 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
       phase = std::min(modTimeLeft[MOD_GLASS] / 2.0, phase);
     else
       phase = 1.0;
-    specular = Color::mix(phase, specular, Color(1., 1., 1., 1.));
+    specular = Color::mix(phase, specular, Color(SRGBColor(1., 1., 1., 1.)));
     sharpness = 50.0 * phase + sharpness * (1.0 - phase);
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);
@@ -412,7 +400,7 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
       phase = std::min(modTimeLeft[MOD_FROZEN] / 2.0, phase);
     else
       phase = 1.0;
-    specular = Color::mix(phase, specular, Color(0.8, 0.8, 1., 1.));
+    specular = Color::mix(phase, specular, Color(SRGBColor(0.8, 0.8, 1., 1.)));
     sharpness = 50.0 * phase + sharpness * (1.0 - phase);
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);
@@ -421,27 +409,46 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
     glEnable(GL_CULL_FACE);
   }
 
+  Coord3d loc((GLfloat)position[0], (GLfloat)position[1], (GLfloat)(position[2] - sink));
+
   {
     int ntries = 0;
     int nverts = 0;
     int detail;
     switch (ballResolution) {
     case BALL_HIRES:
-      detail = 20;
+      detail = 3;
       break;
     default:
     case BALL_NORMAL:
-      detail = 14;
+      detail = 2;
       break;
     case BALL_LORES:
-      detail = 8;
+      detail = 2;
       break;
     }
     countObjectSpherePoints(&ntries, &nverts, detail);
 
+    Matrix3d rotation;
+    if (modTimeLeft[MOD_EXTRA_LIFE]) {
+      Matrix4d ref;
+      identityMatrix(ref);
+      rotateY(M_PI / 3, ref);
+      for (int i = 0; i < 3; i++)
+        for (int j = 0; j < 3; j++) rotation[i][j] = ref[i][j];
+    } else {
+      for (int i = 0; i < 3; i++)
+        for (int j = 0; j < 3; j++) rotation[j][i] = rotations[i][j];
+    }
+
     // Draw the ball...
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(specular, sharpness, Lighting_Regular);
+    Matrix4d transform;
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) { rotation[i][j] *= radius; }
+    }
+    affineMatrix(transform, rotation, loc);
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    setObjectUniforms(uloc, transform, specular, sharpness, Lighting_Regular);
     if (modTimeLeft[MOD_EXTRA_LIFE]) {
       glBindTexture(GL_TEXTURE_2D, textureTrack);
     } else if (texture == 0) {
@@ -465,9 +472,9 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
       glEnable(GL_BLEND);
       glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-      setActiveProgramAndUniforms(Shader_Reflection);
-      glUniformC(uniformLocations.refl_color, c);
-      glUniform1i(uniformLocations.tex, 0);
+      uloc = setActiveProgramAndUniforms(Shader_Reflection);
+      glUniformMatrix4(uloc->object_matrix, transform);
+      glUniformC(uloc->refl_color, c);
       glActiveTexture(GL_TEXTURE0 + 0);
       glBindTexture(GL_TEXTURE_2D, environmentTexture);
 
@@ -480,9 +487,23 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
     glDisable(GL_BLEND);
     glEnable(GL_CULL_FACE);
 
+    GLfloat phase = std::min(modTimePhaseIn[MOD_SPIKE] / 2.0f, 1.0f);
+    if (modTimeLeft[MOD_SPIKE] > 0)
+      phase = std::min(modTimeLeft[MOD_SPIKE] / 2.0f, phase);
+    else
+      phase = 1.0;
+    GLfloat scale = radius * (0.5 + 0.5 * phase);
+
     // Transfer
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(Color(0.1, 0.1, 0.1, 1.), 10.f, Lighting_Regular);
+    Matrix3d rot;
+    Matrix4d transform;
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) { rot[i][j] = rotations[j][i] * scale; }
+    }
+    affineMatrix(transform, rot, loc);
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    setObjectUniforms(uloc, transform, Color(SRGBColor(0.1, 0.1, 0.1, 1.)), 10.f,
+                      Lighting_Regular);
     glBindTexture(GL_TEXTURE_2D, textureBlank);
 
     glBindVertexArray(vaolist[1]);
@@ -495,8 +516,8 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
     glEnable(GL_LINE_SMOOTH);
     glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
 
-    setActiveProgramAndUniforms(Shader_Line);
-    glUniformC(uniformLocations.line_color, Color(1., 1., 1., 0.5));
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Line);
+    glUniformC(uloc->line_color, Color(SRGBColor(1., 1., 1., 0.5)));
 
     const int nlines = 8;
 
@@ -510,8 +531,15 @@ void Ball::drawBuffers1(const GLuint *vaolist) const {
     // In case we look from below
     glDisable(GL_CULL_FACE);
 
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(Color(0.1, 0.1, 0.1, 1.), 10.f, Lighting_Regular);
+    Matrix4d transform;
+    Matrix3d scale;
+    for (int i = 0; i < 3; i++) {
+      for (int j = 0; j < 3; j++) { scale[i][j] = radius * ((i == j) ? 1. : 0.); }
+    }
+    affineMatrix(transform, scale, loc);
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    setObjectUniforms(uloc, transform, Color(SRGBColor(0.1, 0.1, 0.1, 1.)), 10.f,
+                      Lighting_Regular);
     glBindTexture(GL_TEXTURE_2D, textureBlank);
 
     glBindVertexArray(vaolist[3]);
@@ -524,27 +552,43 @@ void Ball::drawBuffers2(const GLuint *vaolist) const {
   if (dontReflectSelf) return;
   if (activeView.calculating_shadows) return;
 
+  Coord3d loc((GLfloat)position[0], (GLfloat)position[1], (GLfloat)(position[2] - sink));
+
   if (modTimeLeft[MOD_EXTRA_LIFE]) {
     glEnable(GL_BLEND);
     glEnable(GL_CULL_FACE);
 
     int ntries = 0;
     int nverts = 0;
-    int detail = 5;
+    int detail = 3;
     countObjectSpherePoints(&ntries, &nverts, detail);
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(Color(0., 0., 0., 0.), 0.f, Lighting_None);
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    Matrix3d scale = {
+        {radius * 1.25, 0., 0.},
+        {0., radius * 1.25, 0.},
+        {0., 0., radius * 1.25},
+    };
+    Matrix4d transform;
+    affineMatrix(transform, scale, loc);
+    setObjectUniforms(uloc, transform, Color(0., 0., 0., 0.), 0.f, Lighting_None);
     glBindTexture(GL_TEXTURE_2D, textureBlank);
 
     glBindVertexArray(vaolist[4]);
     glDrawElements(GL_TRIANGLES, 3 * ntries, GL_UNSIGNED_SHORT, (void *)0);
   }
+
   if (modTimeLeft[MOD_JUMP]) {
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);
 
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(Color(0., 0., 0., 0.), 0.f, Lighting_NoShadows);
+    Matrix4d transform;
+    Matrix3d scale = {{radius, 0, 0}, {0, radius, 0}, {0, 0, radius}};
+    affineMatrix(transform, scale, loc);
+    rotateZ(game.gameTime * 0.15, transform);
+    transform[2][3] += radius * std::fmod(game.gameTime, 1.0);
+
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    setObjectUniforms(uloc, transform, Color(0., 0., 0., 0.), 0.f, Lighting_NoShadows);
     glBindTexture(GL_TEXTURE_2D, textureBlank);
 
     glBindVertexArray(vaolist[5]);
@@ -555,8 +599,13 @@ void Ball::drawBuffers2(const GLuint *vaolist) const {
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);
 
-    setActiveProgramAndUniforms(Shader_Object);
-    setObjectUniforms(Color(0., 0., 0., 0.), 0.f, Lighting_NoShadows);
+    Matrix4d transform;
+    Matrix3d scale = {{radius, 0, 0}, {0, radius, 0}, {0, 0, radius}};
+    affineMatrix(transform, scale, loc);
+    rotateZ(0.5 * M_PI * game.gameTime, transform);
+
+    const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+    setObjectUniforms(uloc, transform, Color(0., 0., 0., 0.), 0.f, Lighting_NoShadows);
     glBindTexture(GL_TEXTURE_2D, textureDizzy);
 
     glBindVertexArray(vaolist[6]);
@@ -775,7 +824,7 @@ bool Ball::physics(Real time) {
         velocity[1] *= 0.9;
         velocity[2] *= 0.9;
         Cell &c = map->cell((int)position[0], (int)position[1]);
-        generateDebris(c.colors[Cell::CENTER]);
+        generateDebris(Color(c.colors[Cell::CENTER]));
       }
     }
   }
@@ -804,7 +853,7 @@ bool Ball::physics(Real time) {
                      velocity[2] * velocity[2] * 5.;
       if (game.frandom() <
           speed * 0.001 * (depth < 0.5 ? depth : 1.0 - depth) * radius / 0.3) {
-        Color waterColor(0.4, 0.4, 0.8, 0.5);
+        Color waterColor(SRGBColor(0.4, 0.4, 0.8, 0.5));
         Coord3d center(position[0], position[1], waterHeight);
         game.add(new Splash(game, center, velocity, waterColor, 30 * radius / 0.3,
                             radius));  // speed*radius*(depth<0.5?depth:1.0-depth)*2.0,radius);
@@ -812,7 +861,7 @@ bool Ball::physics(Real time) {
       // splashes caused by rotation. eg "swimming"
       speed = rotation[0] * rotation[0] + rotation[1] * rotation[1];
       if (game.frandom() < speed * 0.001) {
-        Color waterColor(0.4, 0.4, 0.8, 0.5);
+        Color waterColor(SRGBColor(0.4, 0.4, 0.8, 0.5));
         Coord3d center(position[0], position[1], waterHeight);
         Coord3d vel;
         vel[0] = -rotation[0] * radius;  // 0.3;
@@ -840,7 +889,7 @@ bool Ball::physics(Real time) {
     double speed2 =
         velocity[0] * velocity[0] + velocity[1] * velocity[1] + velocity[2] * velocity[2];
     if (game.frandom() < (speed2 - 0.2) * 0.05) {
-      Color acidColor(0.1, 0.5, 0.1, 0.5);
+      Color acidColor(SRGBColor(0.1, 0.5, 0.1, 0.5));
       Coord3d center(position[0], position[1], mapHeight);
       game.add(new Splash(game, center, velocity, acidColor, speed2 * radius, radius));
     }
@@ -1162,7 +1211,7 @@ bool Ball::handleGround(Map *map, Cell **cells, Coord3d *hitpts, Coord3d *normal
 
     /* Acid splash */
     if (nacidsplash) {
-      Color acidColor(0.1, 0.5, 0.1, 0.5);
+      Color acidColor(SRGBColor(0.1, 0.5, 0.1, 0.5));
       Coord3d center(position[0], position[1], map->getHeight(position[0], position[1]));
       game.add(new Splash(game, center, velocity, acidColor,
                           (acidSpeed / nacidsplash) * radius * 20.0, radius));
@@ -1443,8 +1492,8 @@ void Ball::generateSandDebris() {
   Debris *d = new Debris(game, NULL, pos, vel, 0.5 + 1.0 * game.frandom());
   game.add(d);
   d->initialSize = 0.05;
-  d->primaryColor = Color(0.6 + 0.3 * game.frandom(), 0.5 + 0.4 * game.frandom(),
-                          0.1 + 0.3 * game.frandom(), 1.f);
+  d->primaryColor = Color(SRGBColor(0.6 + 0.3 * game.frandom(), 0.5 + 0.4 * game.frandom(),
+                                    0.1 + 0.3 * game.frandom(), 1.f));
   d->friction = 0.0;
   d->calcRadius();
 }
@@ -1460,7 +1509,7 @@ void Ball::generateNitroDebris(Real time) {
     d->velocity[2] += 0.2;
     d->gravity = -0.1;
     d->modTimeLeft[MOD_GLASS] = -1.0;
-    d->primaryColor = Color(0.1, 0.6, 0.1, 1.0);
+    d->primaryColor = Color(SRGBColor(0.1, 0.6, 0.1, 1.0));
     d->no_physics = true;
   }
 }
