@@ -232,7 +232,7 @@ EditMode::EditMode() {
   scale = 0.5;
   rotation = 0.0;
   raise = 0.5;
-  color = Color(0.9, 0.9, 0.9, 1.0);
+  color = SRGBColor(0.9, 0.9, 0.9, 1.0);
   doSave = 0;
   cellClipboard = NULL;
 
@@ -380,10 +380,9 @@ void EditMode::display() {
     if (Settings::settings->doShadows) {
       Coord3d focus((double)x, (double)y, map->getHeight(x, y));
       renderShadowCascade(focus, map, game);
-      renderDummyShadowMap();
+      activeView.use_shadows = true;
     } else {
-      renderDummyShadowCascade();
-      renderDummyShadowMap();
+      activeView.use_shadows = false;
     }
   }
 
@@ -402,7 +401,7 @@ void EditMode::display() {
     map->drawLoop(0, 0, map->width - 1, map->height - 1, 0);
     if (game) {
       /* Indicate start position and trigger object locations */
-      for (int i = 0; i < game->hooks[Role_GameHook].size(); i++) {
+      for (size_t i = 0; i < game->hooks[Role_GameHook].size(); i++) {
         GameHook* hook = game->hooks[Role_GameHook][i];
         if (hook->invalid) continue;
 
@@ -485,12 +484,26 @@ typedef enum { eInfo_Height = 0, eInfo_WaterHeight, eInfo_Textures, nInfos } eCe
 int cellInfo = 0;
 char* infoNames[nInfos] = {_("Cell height"), _("Cell water height"), _("Textures")};
 
-int roundint(double f) {
-  /* std::round is C++11. */
-  return std::floor(f + 0.5);
+void EditMode::mouseDown(int state, int x, int y) { MyWindow::mouseDownAll(state, x, y); }
+
+static uint16_t incrementColor(uint16_t srgb, bool decrease) {
+  if (decrease) {
+    if (srgb == 0)
+      return MAX_SRGB_VAL;
+    else if (srgb <= MAX_SRGB_VAL / 20)
+      return 0;
+    else
+      return srgb - MAX_SRGB_VAL / 20;
+  } else {
+    if (srgb >= MAX_SRGB_VAL)
+      return 0;
+    else if (srgb >= MAX_SRGB_VAL - MAX_SRGB_VAL / 20)
+      return MAX_SRGB_VAL;
+    else
+      return srgb + MAX_SRGB_VAL / 20;
+  }
 }
 
-void EditMode::mouseDown(int state, int x, int y) { MyWindow::mouseDownAll(state, x, y); }
 void EditMode::doCommand(int command) {
   int mouseX, mouseY;
   int mouseState;
@@ -538,6 +551,7 @@ void EditMode::doCommand(int command) {
     yLow = yHigh = y;
   }
 
+  bool not_lbutton = (mouseState & SDL_BUTTON_LMASK) ? 0 : 1;
   switch (command) {
   case EDIT_HEIGHT:
   case EDIT_COLOR:
@@ -556,47 +570,35 @@ void EditMode::doCommand(int command) {
     break;
 
   case EDIT_UPPER:
-    doCellAction(CODE_CELL_N, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_N, not_lbutton);
     break;
   case EDIT_BOTTOM:
-    doCellAction(CODE_CELL_S, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_S, not_lbutton);
     break;
   case EDIT_LEFT:
-    doCellAction(CODE_CELL_W, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_W, not_lbutton);
     break;
   case EDIT_RIGHT:
-    doCellAction(CODE_CELL_E, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_E, not_lbutton);
     break;
   case EDIT_ALL:
-    doCellAction(CODE_CELL_ALL, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_ALL, not_lbutton);
     break;
   case EDIT_CENTER:
-    doCellAction(CODE_CELL_C, mouseState & SDL_BUTTON_LMASK ? 0 : 1);
+    doCellAction(CODE_CELL_C, not_lbutton);
     break;
 
   case COLOR_RED:
-    if (color.v[0] >= 65535)
-      color.v[0] = 0;
-    else
-      color.v[0] = std::min(65535, color.v[0] + 3277);
+    color.w[0] = incrementColor(color.w[0], not_lbutton);
     break;
   case COLOR_GREEN:
-    if (color.v[1] >= 65535)
-      color.v[1] = 0;
-    else
-      color.v[1] = std::min(65535, color.v[1] + 3277);
+    color.w[1] = incrementColor(color.w[1], not_lbutton);
     break;
   case COLOR_BLUE:
-    if (color.v[2] >= 65535)
-      color.v[2] = 0;
-    else
-      color.v[2] = std::min(65535, color.v[2] + 3277);
+    color.w[2] = incrementColor(color.w[2], not_lbutton);
     break;
   case COLOR_ALPHA:
-    if (color.v[3] >= 65535)
-      color.v[3] = 0;
-    else
-      color.v[3] = std::min(65535, color.v[3] + 3277);
+    color.w[3] = incrementColor(color.w[3], not_lbutton);
     break;
 
   case FLAG_0:
@@ -664,7 +666,7 @@ void EditMode::doCommand(int command) {
                 val = map->cell(altx, alty).waterHeights[m];
                 break;
               case REPAIR_COLOR_CONT:
-                val = map->cell(altx, alty).colors[m].v[p];
+                val = map->cell(altx, alty).colors[m].w[p];
               }
               if (xLow <= altx && altx <= xHigh && yLow <= alty && alty <= yHigh) {
                 internal += val;
@@ -700,8 +702,8 @@ void EditMode::doCommand(int command) {
               c.waterHeights[k] = target;
               break;
             case REPAIR_COLOR_CONT:
-              dcent += target - c.colors[k].v[p];
-              c.colors[k].v[p] = target;
+              dcent += target - c.colors[k].w[p];
+              c.colors[k].w[p] = target;
             }
           }
           switch (command) {
@@ -713,7 +715,7 @@ void EditMode::doCommand(int command) {
             c.waterHeights[4] += dcent / 4;
             break;
           case REPAIR_COLOR_CONT:
-            c.colors[4].v[p] += dcent / 4;
+            c.colors[4].w[p] += dcent / 4;
           }
         }
     }
@@ -742,8 +744,8 @@ void EditMode::doCommand(int command) {
           break;
         case REPAIR_COLOR_CENTERS:
           for (int k = 0; k < 4; k++) {
-            c.colors[Cell::CENTER].v[k] =
-                (c.colors[0].v[k] + c.colors[1].v[k] + c.colors[2].v[k] + c.colors[3].v[k]) /
+            c.colors[Cell::CENTER].w[k] =
+                (c.colors[0].w[k] + c.colors[1].w[k] + c.colors[2].w[k] + c.colors[3].w[k]) /
                 4;
           }
           break;
@@ -765,12 +767,12 @@ void EditMode::doCommand(int command) {
           break;
         case REPAIR_COLOR_ROUND:
           for (int m = 0; m < 4; m++) {
-            float mscale = 0.05 * 65535.f;
+            float mscale = MAX_SRGB_VAL / 20;
             for (int k = 0; k < 4; k++) {
-              c.colors[k].v[m] = mscale * roundint(c.colors[k].v[m] / mscale);
+              c.colors[k].w[m] = mscale * roundint(c.colors[k].w[m] / mscale);
             }
-            c.colors[Cell::CENTER].v[m] =
-                0.25 * mscale * roundint(4. * c.colors[Cell::CENTER].v[m] / mscale);
+            c.colors[Cell::CENTER].w[m] =
+                0.25 * mscale * roundint(4. * c.colors[Cell::CENTER].w[m] / mscale);
           }
           break;
         case REPAIR_VEL_ROUND:
