@@ -24,20 +24,16 @@
 #include "game.h"
 
 #include "animatedCollection.h"
-#include "ball.h"
-#include "forcefield.h"
+#include "flag.h"
 #include "gamer.h"
 #include "guile.h"
 #include "map.h"
-#include "pipe.h"
 #include "player.h"
 #include "replay.h"
 #include "settings.h"
 #include "sound.h"
 #include "weather.h"
 
-Game *Game::current = NULL;
-
 extern Color colors[5];
 
 double Game::defaultScores[SCORE_MAX][2];
@@ -56,11 +52,12 @@ Game::Game(const char *name, Gamer *gamer) {
   edit_mode = 0;
 
   weather = new Weather();
+  flagRenderer = new FlagRenderer();
 
   /* Load the bootup script */
   char scmname[256];
   snprintf(scmname, sizeof(scmname), "%s/levels/boot.scm", effectiveShareDir);
-  loadScript(scmname);
+  loadScript(this, scmname);
 
   player1 = new Player(*this);
   add(player1);
@@ -80,6 +77,7 @@ Game::Game(Map *editmap, const char *levelname) {
   gameTicks = 0;
   nextLevel[0] = 0;
   weather = NULL;
+  flagRenderer = new FlagRenderer();
   map = editmap;
   edit_mode = 1;
   setDefaults();
@@ -88,14 +86,14 @@ Game::Game(Map *editmap, const char *levelname) {
   char scmname[512];
   snprintf(scmname, sizeof(scmname), "%s/levels/boot.scm", effectiveShareDir);
   scmname[511] = '\0';
-  loadScript(scmname);
+  loadScript(this, scmname);
   snprintf(scmname, 511, "%s/levels/%s.scm", effectiveLocalDir, levelname);
   scmname[511] = '\0';
   if (!fileExists(scmname)) {
     snprintf(scmname, 511, "%s/levels/%s.scm", effectiveShareDir, levelname);
   }
   scmname[511] = '\0';
-  loadScript(scmname);
+  loadScript(this, scmname);
 
   for (int j = 0; j < newHooks.size(); j++) {
     hooks[newHooks[j]->entity_role].push_back(newHooks[j]);
@@ -108,6 +106,7 @@ Game::~Game() {
 
   delete weather;
   delete balls;
+  delete flagRenderer;
   if (player1) {
     if (hooks[Role_Player].size() && hooks[Role_Player][0] == player1) {
       hooks[Role_Player].clear();
@@ -129,7 +128,7 @@ void Game::loadLevel(const char *name, Gamer *gamer) {
   } else {
     /* when there is no gamer (e.g, in help) pick random value */
     player1->primaryColor = Color(frandom(), frandom(), frandom(), 1.0);
-    player1->primaryColor.v[((size_t)rand()) % 3] = 1.0;
+    player1->primaryColor.v[((size_t)rand()) % 3] = 1.f;
     player1->texture = loadTexture("blank.png");
   }
 
@@ -167,9 +166,7 @@ void Game::loadLevel(const char *name, Gamer *gamer) {
   if (map) delete map;
   map = new Map(mapname);
 
-  Game::current = this;
-  loadScript(scmname);
-  Game::current = NULL;
+  loadScript(this, scmname);
 
   for (int j = 0; j < newHooks.size(); j++) {
     hooks[newHooks[j]->entity_role].push_back(newHooks[j]);
@@ -277,14 +274,13 @@ void Game::handleKey(int key) {
 void *Game::runQueuedCalls(void *data) {
   Game *game = (Game *)data;
 
-  Game::current = game;
   for (int j = 0; j < game->queuedCalls.size(); j++) {
     QueuedCall call = game->queuedCalls[j];
     /* the functions are owned by GameHooks; arguments by Game */
     if (call.type == 0) {
-      scm_catch_call_0(call.fun);
+      scm_catch_call_0(game, call.fun);
     } else if (call.type == 1) {
-      scm_catch_call_1(call.fun, scm_from_double(call.val));
+      scm_catch_call_1(game, call.fun, scm_from_double(call.val));
     } else if (call.type == 2) {
       SCM sub;
       Animated *a = dynamic_cast<Animated *>(call.subject);
@@ -293,12 +289,11 @@ void *Game::runQueuedCalls(void *data) {
       else
         sub = smobGameHook_make(call.subject);
 
-      scm_catch_call_2(call.fun, sub, call.object ? call.object : SCM_BOOL_F);
+      scm_catch_call_2(game, call.fun, sub, call.object ? call.object : SCM_BOOL_F);
       if (call.object) { scm_gc_unprotect_object(call.object); }
     }
   }
 
-  Game::current = NULL;
   game->queuedCalls.clear();
 
   return NULL;
@@ -393,12 +388,14 @@ void Game::draw() {
 
   /* Draw first pass of all objects */
   for (int i = Role_OtherAnimated; i < Role_MaxTypes; i++) {
+    if (i == Role_Flag) { continue; }
     int n = hooks[i].size();
     for (int j = 0; j < n; j++) {
       Animated *anim = (Animated *)hooks[i][j];
       if (anim->onScreen) anim->draw();
     }
   }
+  flagRenderer->draw(hooks[Role_Flag]);
 
   /* Draw second pass of all objects */
   for (int i = Role_OtherAnimated; i < Role_MaxTypes; i++) {
