@@ -32,21 +32,27 @@ typedef struct SDL_Surface SDL_Surface;
 class Map;
 class Game;
 
+extern const Matrix4d identity4;
+extern const Matrix3d identity3;
+
 /* Prototypes */
 void glHelpInit();
 void glHelpCleanup();
 void warnForGLerrors(const char *where_am_i);
 
+/* renders and caches a drawing of a 2d string; returns the recommended width
+ * to draw it at. */
+int prepare2DString(TTF_Font *font, const char *string, bool outlined);
+
 /* displays a 2d text on specific screen coordinates, returning width */
-int draw2DString(TTF_Font *, const char *, int x, int y, float red, float green, float blue,
-                 float alpha, bool outlined, int align, int maxwidth = 0);
+int draw2DString(TTF_Font *, const char *, int x, int y, Color color, bool outlined, int align,
+                 int maxwidth = 0);
 void update2DStringCache(bool force_wipe);
 /* Common interface for drawing 2d things on the screen */
 void draw2DRectangle(GLfloat x, GLfloat y, GLfloat w, GLfloat h, GLfloat tx, GLfloat ty,
-                     GLfloat tw, GLfloat th, GLfloat r, GLfloat g, GLfloat b, GLfloat a,
-                     GLuint tex = 0);
-void draw2DQuad(const GLfloat corners[4][2], const GLfloat texture[4][2],
-                const GLfloat color[4][4], GLuint tex = 0);
+                     GLfloat tw, GLfloat th, Color color, GLuint tex = 0);
+void draw2DQuad(const GLfloat corners[4][2], const GLfloat texture[4][2], const Color color[4],
+                GLuint tex = 0);
 
 void tickMouse(Real td);
 void drawMousePointer();
@@ -67,8 +73,8 @@ inline uint32_t packNormal(const GLfloat n[3]) {
   return x;
 }
 void countObjectSpherePoints(int *ntriangles, int *nvertices, int detail);
-void placeObjectSphere(void *data, ushort *idxs, ushort first_index, GLfloat const position[3],
-                       Matrix3d rotation, GLfloat radius, int detail, const Color &color);
+void placeObjectSphere(void *data, ushort *idxs, ushort first_index, int detail,
+                       const Color &color);
 
 typedef enum {
   Shader_Tile,
@@ -79,36 +85,79 @@ typedef enum {
 } Shader_Type;
 /* call markViewChanged when activeView is changed */
 void markViewChanged();
-/* setActiveProgramAndUniforms switches program, sets view uniforms, and updates common uniform
- * locations */
-void setActiveProgramAndUniforms(Shader_Type shader);
+
+/* This struct provides the uniform locations for a program.
+ * If the program does not have a uniform, the corresponding location is
+ * filled in with '-1' */
+typedef struct {
+  /* general 3D */
+  GLint mvp_matrix;
+  GLint model_matrix;
+  /* common */
+  GLint gameTime;
+
+  /* line */
+  GLint line_color;
+  /* tile */
+  GLint arrtex;
+  /* water */
+  GLint wtex;
+
+  /* object */
+  GLint specular;
+  GLint sharpness;
+  GLint ignore_shadow;
+  GLint use_lighting;
+
+  /* reflection */
+  GLint refl_color;
+
+  /* reflection, object, ui */
+  GLint tex;
+
+  /* all display */
+  GLint fog_active;
+  GLint fog_color;
+  GLint fog_start;
+  GLint fog_end;
+
+  /* object/water/tile */
+  GLint light_ambient;
+  GLint light_diffuse;
+
+  /* object */
+  GLint light_specular;
+  GLint object_matrix;
+
+  /* day */
+  GLint shadow_cascade0;
+  GLint shadow_cascade1;
+  GLint shadow_cascade2;
+  GLint cascade_mvp;
+  GLint cascade_model;
+  GLint shadowtex_size;
+  GLint sun_direction;
+
+  /* night */
+  GLint shadow_map;
+  GLint light_position;
+
+  // For UI shader
+  GLint screen_width;
+  GLint screen_height;
+} UniformLocations;
+
+/* setActiveProgramAndUniforms switches program, sets view uniforms,
+ * and returns the uniform location map for the program. */
+const UniformLocations *setActiveProgramAndUniforms(Shader_Type shader);
 /* update object shader uniforms */
 typedef enum {
   Lighting_None,      /* No shadows, orientation independent */
   Lighting_NoShadows, /* Orientation dependent colors, no shadows */
   Lighting_Regular    /* Shadows and orientation dependent colors */
 } Object_Lighting;
-void setObjectUniforms(Color specular, float sharpness, Object_Lighting lighting);
-/* if the currently active shader has a uniform with matching name, then this struct provides
- * the location */
-struct UniformLocations {
-  GLuint line_color;
-  GLuint refl_color;
-  GLuint tex;
-  GLuint render_stage;
-  GLuint gameTime;
-  GLuint arrtex;
-  GLuint wtex;
-  GLuint specular_color;
-  GLuint sharpness;
-  GLuint ignore_shadow;
-  GLuint use_lighting;
-  // For UI shader
-  GLuint ui_screen_width;
-  GLuint ui_screen_height;
-  GLuint ui_tex;
-};
-extern UniformLocations uniformLocations;
+void setObjectUniforms(const UniformLocations *locations, const Matrix4d object_transform,
+                       Color specular, float sharpness, Object_Lighting lighting);
 
 typedef struct _viewpa {
   Matrix4d modelview;
@@ -126,6 +175,7 @@ typedef struct _viewpa {
   GLfloat light_position[3];
   GLfloat sun_direction[3];
 
+  bool use_shadows;
   bool day_mode;
   GLuint cascadeTexsize;
   GLuint cascadeTexture[3];
@@ -147,8 +197,6 @@ void lookAtMatrix(GLdouble eyeX, GLdouble eyeY, GLdouble eyeZ, GLdouble centerX,
                   GLdouble centerY, GLdouble centerZ, GLdouble upX, GLdouble upY, GLdouble upZ,
                   Matrix4d out);
 
-void renderDummyShadowCascade();
-void renderDummyShadowMap();
 void renderShadowMap(const Coord3d &focus, Map *mp, Game *gm);
 void renderShadowCascade(const Coord3d &focus, Map *mp, Game *gm);
 
@@ -167,6 +215,7 @@ void debugMatrix(Matrix4d);
 Coord3d useMatrix(Matrix4d, const Coord3d &);
 Coord3d useMatrix(Matrix3d, const Coord3d &);
 void identityMatrix(Matrix4d);
+void affineMatrix(Matrix4d, const Matrix3d rot, const Coord3d &pos);
 void assign(const Matrix4d, Matrix4d);
 void matrixMult(const Matrix4d, const Matrix4d, Matrix4d);
 void rotateX(double, Matrix4d);
@@ -210,7 +259,7 @@ extern float fps;
 extern int screenWidth, screenHeight;
 extern TTF_Font *ingameFont;
 TTF_Font *menuFontForSize(int sz);
-extern const GLfloat menuColorSelected[4], menuColor[4];
+extern const Color menuColorSelected, menuColor;
 
 /***********************************/
 /*  Inlined vector operations      */
@@ -237,4 +286,12 @@ inline double dotProduct(const Coord3d &A, const Coord3d &B) {
 inline void glUniformC(GLint l, const Color &c) {
   glUniform4f(l, c.f0(), c.f1(), c.f2(), c.f3());
 }
+
+inline void glUniformMatrix4(GLint l, const Matrix4d mtx) {
+  GLfloat lobject[16];
+  for (int i = 0; i < 4; i++) {
+    for (int j = 0; j < 4; j++) { lobject[4 * i + j] = mtx[j][i]; }
+  }
+  glUniformMatrix4fv(l, 1, GL_FALSE, (GLfloat *)lobject);
+}
 #endif
