@@ -26,8 +26,8 @@
 
 class HeightModifier : public GameHook {
  public:
-  HeightModifier(int corner, int x, int y, Real min, Real max, Real freq, Real phase, int not1,
-                 int not2, int not3);
+  HeightModifier(Game& g, int corner, int x, int y, Real min, Real max, Real freq, Real phase,
+                 int not1, int not2, int not3);
 
   void tick(Real t);
 
@@ -36,6 +36,7 @@ class HeightModifier : public GameHook {
   int corner, x, y;
   int avg_center;
   int not1, not2, not3;
+  Real localtime;
 };
 
 #endif
