@@ -53,9 +53,7 @@ HelpMode::HelpMode() {
   helpGame = NULL;
   if (low_memory)
     background = NULL;
-  else {
-    background = loadImage("helpBackground.jpg");
-  }
+  else { background = loadImage("helpBackground.jpg"); }
 }
 HelpMode::~HelpMode() {
   if (background) SDL_FreeSurface(background);
@@ -93,7 +91,7 @@ void HelpMode::display() {
 
   Enter2DMode();
   draw2DRectangle(0, 0, screenWidth, screenHeight, bgCoord[0], bgCoord[1], bgCoord[2],
-                  bgCoord[3], 1., 1., 1., 1., bgTexture);
+                  bgCoord[3], Color(1., 1., 1., 1.), bgTexture);
   Leave2DMode();
 
   /* for shadow map */
@@ -146,10 +144,10 @@ void HelpMode::display() {
   markViewChanged();
   if (Settings::settings->doShadows) {
     renderShadowCascade(cameraTo1, helpGame->map, helpGame);
+    activeView.use_shadows = true;
   } else {
-    renderDummyShadowCascade();
+    activeView.use_shadows = false;
   }
-  renderDummyShadowMap();
 
   glViewport(11 * screenWidth / 20, 11 * screenHeight / 20, 4 * screenWidth / 10,
              4 * screenHeight / 10);
@@ -164,10 +162,10 @@ void HelpMode::display() {
   markViewChanged();
   if (Settings::settings->doShadows) {
     renderShadowCascade(cameraTo1, helpGame->map, helpGame);
+    activeView.use_shadows = true;
   } else {
-    renderDummyShadowCascade();
+    activeView.use_shadows = false;
   }
-  renderDummyShadowMap();
   glViewport(11 * screenWidth / 20, 1 * screenHeight / 10, 4 * screenWidth / 10,
              4 * screenHeight / 10);
   Coord3d focus_b(step / 2 + step * page, 3 * step / 2, 2);
@@ -265,14 +263,14 @@ void HelpMode::display() {
   if (s1 >= 0) {
     for (int i = s1; text[i]; i++) {
       drawSimpleText(text[i], 50, 1 * screenHeight / 10 + (i - s1) * 3 * fontSize + fontSize,
-                     fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
+                     fontSize, Color(SRGBColor(0.9, 0.9, 0.25, 1.)), screenWidth / 2 - 50);
     }
   }
 
   if (s2 >= 0) {
     for (int i = s2; text[i]; i++) {
       drawSimpleText(text[i], 50, 6 * screenHeight / 10 + (i - s2) * 3 * fontSize + fontSize,
-                     fontSize, 0.9, 0.9, 0.25, 1., screenWidth / 2 - 50);
+                     fontSize, Color(SRGBColor(0.9, 0.9, 0.25, 1.)), screenWidth / 2 - 50);
     }
   }
 
