@@ -109,23 +109,15 @@ void MainMode::display() {
   /* lighting must be set before we render the shadow map */
   /* Shadow map rendering returns active modelview/projection to orig state */
   setupLighting(game, game->isNight);
-  if (game->isNight) {
-    if (Settings::settings->doShadows) {
+  if (Settings::settings->doShadows) {
+    if (game->isNight) {
       renderShadowMap(camFocus, map, game);
-      renderDummyShadowCascade();
     } else {
-      renderDummyShadowCascade();
-      renderDummyShadowMap();
-    }
-  } else {
-    activeView.day_mode = true;
-    if (Settings::settings->doShadows) {
       renderShadowCascade(camFocus, map, game);
-      renderDummyShadowMap();
-    } else {
-      renderDummyShadowCascade();
-      renderDummyShadowMap();
     }
+    activeView.use_shadows = true;
+  } else {
+    activeView.use_shadows = false;
   }
 
   /* record modelview/projection matrices for item visibility testing */
@@ -217,9 +209,10 @@ void MainMode::display() {
 
     Enter2DMode();
     // darken the display
-    draw2DRectangle(0, 0, screenWidth, screenHeight, 0., 0., 1., 1., 0., 0., 0., pause_time);
-    drawCenterSimpleText(_("Paused"), screenWidth / 2, screenHeight / 2 - 16, 16, 1.0, 1.0,
-                         1.0, 0.75);
+    draw2DRectangle(0, 0, screenWidth, screenHeight, 0., 0., 1., 1.,
+                    Color(0., 0., 0., pause_time));
+    drawCenterSimpleText(_("Paused"), screenWidth / 2, screenHeight / 2 - 16, 16,
+                         Color(1.0, 1.0, 1.0, 0.75));
     Leave2DMode();
     break;
   case statusBonusLevelComplete:
@@ -254,8 +247,10 @@ void MainMode::display() {
 
   Enter2DMode();
   if (player1->modTimeLeft[MOD_FROZEN]) {
-    draw2DRectangle(0, 0, screenWidth, screenHeight, 0., 0., 1., 1., 0.5, 0.5, 1.0,
-                    0.5 * std::min(1.0, (double)player1->modTimeLeft[MOD_FROZEN]));
+    draw2DRectangle(
+        0, 0, screenWidth, screenHeight, 0., 0., 1., 1.,
+        Color(SRGBColor(0.5, 0.5, 1.0,
+                        0.5 * std::min(1.0, (double)player1->modTimeLeft[MOD_FROZEN]))));
   }
   /* Print joystick debugging information */
   if (debug_joystick && Settings::settings->hasJoystick()) {
@@ -263,7 +258,8 @@ void MainMode::display() {
     snprintf(str, 255, "Joy: %d, %d -> %.1f, %.1f", Settings::settings->joystickRawX(),
              Settings::settings->joystickRawY(), Settings::settings->joystickX(),
              Settings::settings->joystickY());
-    drawCenterSimpleText(str, screenWidth / 2, screenHeight - 16, 8, 0.6, 0.6, 0.6, 0.6);
+    drawCenterSimpleText(str, screenWidth / 2, screenHeight - 16, 8,
+                         Color(SRGBColor(0.6, 0.6, 0.6, 0.6)));
   }
 
   if (debug_shadowmap) {
@@ -331,7 +327,7 @@ void MainMode::display() {
 
       int s = 128;
       draw2DRectangle(50 + (s + 10) * locations[i][0], 50 + (s + 10) * locations[i][1], s, s,
-                      0., 1., 1., -1., 0.9, 0.8, 0.8, 1., cpy);
+                      0., 1., 1., -1., Color(SRGBColor(0.9, 0.8, 0.8, 1.)), cpy);
 
       glDeleteTextures(1, &cpy);
     }
@@ -348,10 +344,10 @@ void MainMode::doExpensiveComputations() { game->doExpensiveComputations(); }
 void MainMode::key(int key) {
   switch (gameStatus) {
   case statusBeforeGame:
-    if (key == ' ') startGame();
+    if (key == SDLK_SPACE || key == SDLK_RETURN) startGame();
     break;
   case statusRestartPlayer:
-    if (key == ' ') {
+    if (key == SDLK_SPACE || key == SDLK_RETURN) {
       restartPlayer();
       gameStatus = statusInGame;
     }
@@ -377,7 +373,7 @@ void MainMode::key(int key) {
     break;
   case statusBonusLevelComplete:
   case statusNextLevel:
-    if (key == ' ') {
+    if (key == SDLK_SPACE || key == SDLK_RETURN) {
       if (!game->nextLevel[0]) {
         gameStatus = statusVictory;
         break;
@@ -392,7 +388,7 @@ void MainMode::key(int key) {
     }
     break;
   case statusGameOver: {
-    if (key == ' ' || key == 'n') {
+    if (key == SDLK_SPACE || key == SDLK_RETURN || key == 'n') {
       EnterHighScoreMode::init();
       EnterHighScoreMode::enterHighScoreMode->lastGameScore = game->player1->score;
       EnterHighScoreMode::enterHighScoreMode->lastLevelSet = game->currentLevelSet;
@@ -400,7 +396,7 @@ void MainMode::key(int key) {
     }
   } break;
   case statusVictory:
-    if (key == ' ') {
+    if (key == SDLK_SPACE || key == SDLK_RETURN) {
       Gamer::gamer->playerLose(game);
       EnterHighScoreMode::init();
       EnterHighScoreMode::enterHighScoreMode->lastGameScore = game->player1->score;
@@ -538,7 +534,7 @@ void MainMode::mouse(int state, int x, int y) {
   if (game) game->player1->mouse(state, x, y);
 }
 void MainMode::mouseDown(int button, int /*x*/, int /*y*/) {
-  if (button == SDL_BUTTON_LMASK) key(' ');
+  if (button == SDL_BUTTON_LMASK) key(SDLK_SPACE);
 }
 /* Starts the current game */
 void MainMode::startGame() {
@@ -582,9 +578,10 @@ void MainMode::showInfo() {
   GLfloat notexco[4][2] = {{0., 0.}, {0., 0.}, {0., 0.}, {0., 0.}};
 
   GLfloat pvertices[4][2] = {{0.f, 0.f}, {160.f, 0.f}, {0.f, 100.f}, {160.f, 100.f}};
-  GLfloat pcols[4][4] = {
-      {1.0, 1.0, 0.0, 1.0}, {1.0, 0.5, 0.1, 1.0}, {1.0, 0.5, 0.1, 1.0}, {1.0, 0.1, 0.1, 1.0}};
-  draw2DRectangle(0., 0., 160. + 1.5, 100. + 1.5, 0., 0., 1., 1., 0., 0., 0., 1.);
+  Color pcols[4] = {Color(SRGBColor(1.0, 1.0, 0.0, 1.0)), Color(SRGBColor(1.0, 0.5, 0.1, 1.0)),
+                    Color(SRGBColor(1.0, 0.5, 0.1, 1.0)),
+                    Color(SRGBColor(1.0, 0.1, 0.1, 1.0))};
+  draw2DRectangle(0., 0., 160. + 1.5, 100. + 1.5, 0., 0., 1., 1., Color(0., 0., 0., 1.));
   draw2DQuad(pvertices, notexco, pcols);
 
   const char *fieldNames[3] = {_("Lives"), _("Score"), _("Time left")};
@@ -594,49 +591,49 @@ void MainMode::showInfo() {
                                {152.f, 13.f + i * s},
                                {8.f, 35.f + i * s},
                                {152.f, 35.f + i * s}};
-    GLfloat mcols[4][4] = {{0.75, 0.5, 0.0, 1.0},
-                           {1.0, 0.25, 0.0, 1.0},
-                           {0.75, 0.25, 0.0, 1.0},
-                           {1.0, 0.0, 0.0, 1.0}};
-    draw2DRectangle(8.f - 1.5f, 13.f + i * s - 1.5f, 144.f, 22.f, 0., 0., 1., 1., 0., 0., 0.,
-                    1.);
+    Color mcols[4] = {
+        Color(SRGBColor(0.75, 0.5, 0.0, 1.0)), Color(SRGBColor(1.0, 0.25, 0.0, 1.0)),
+        Color(SRGBColor(0.75, 0.25, 0.0, 1.0)), Color(SRGBColor(1.0, 0.0, 0.0, 1.0))};
+    draw2DRectangle(8.f - 1.5f, 13.f + i * s - 1.5f, 144.f, 22.f, 0., 0., 1., 1.,
+                    Color(0., 0., 0., 1.));
     draw2DQuad(mvertices, notexco, mcols);
-    drawSimpleText(fieldNames[i], 12, 23 + i * s, 9, 1.0, 1.0, 0.0, 1.0);
+    drawSimpleText(fieldNames[i], 12, 23 + i * s, 9, Color(SRGBColor(1.0, 1.0, 0.0, 1.0)));
   }
 
   // lives
   for (int i = 0; i < 4; i++) {
     const char *name = (i < player->lives) ? "life.png" : "nolife.png";
-    draw2DRectangle(63 + i * 20, 7, 32, 32, 0., 0., 1., 1., 1., 1., 1., 1.,
+    draw2DRectangle(63 + i * 20, 7, 32, 32, 0., 0., 1., 1., Color(1., 1., 1., 1.),
                     textures[loadTexture(name)]);
   }
 
   // Score
   char str[256];
   snprintf(str, sizeof(str), "%d", player->score);
-  drawRightSimpleText(str, 150, 52, 9, 0.0, 0.0, 0.0, 1.0);
+  drawRightSimpleText(str, 150, 52, 9, Color(0.0, 0.0, 0.0, 1.0));
 
   // Time left
   snprintf(str, sizeof(str), "%2.2d:%2.2d", player->timeLeft / 60, player->timeLeft % 60);
   if (player->timeLeft < 15 && player->timeLeft % 2)
-    drawRightSimpleText(str, 150, 81, 9, 1.0, 1.0, 1.0, 1.0);
+    drawRightSimpleText(str, 150, 81, 9, Color(1.0, 1.0, 1.0, 1.0));
   else
-    drawRightSimpleText(str, 150, 81, 9, 0.0, 0.0, 0.0, 1.0);
+    drawRightSimpleText(str, 150, 81, 9, Color(0.0, 0.0, 0.0, 1.0));
 
   // Health
-  draw2DRectangle(8, 3, 144, 5, 0., 0., 1., 1., 0.5, 0.1, 0.1, 1.0);
+  draw2DRectangle(8, 3, 144, 5, 0., 0., 1., 1., Color(SRGBColor(0.5, 0.1, 0.1, 1.0)));
 
   GLfloat bvertices[4][2] = {{8.f, 3.f},
                              {8.f + (144.f * (GLfloat)player->health), 3.f},
                              {8.f, 8.f},
                              {8.f + (144.f * (GLfloat)player->health), 8.f}};
-  GLfloat bcols[4][4] = {
-      {0.8, 0.8, 0.1, 1.0}, {0.8, 0.1, 0.1, 1.0}, {0.8, 0.8, 0.1, 1.0}, {0.8, 0.1, 0.1, 1.0}};
+  Color bcols[4] = {Color(SRGBColor(0.8, 0.8, 0.1, 1.0)), Color(SRGBColor(0.8, 0.1, 0.1, 1.0)),
+                    Color(SRGBColor(0.8, 0.8, 0.1, 1.0)),
+                    Color(SRGBColor(0.8, 0.1, 0.1, 1.0))};
   draw2DQuad(bvertices, notexco, bcols);
 
   // Oxygen
-  draw2DRectangle(8, 3, (int)(144.0 * (1. - player->oxygen)), 5, 0., 0., 1., 1., 0.2, 0.2, 0.7,
-                  0.8);
+  draw2DRectangle(8, 3, (int)(144.0 * (1. - player->oxygen)), 5, 0., 0., 1., 1.,
+                  Color(SRGBColor(0.2, 0.2, 0.7, 0.8)));
 
   Leave2DMode();
 }
@@ -758,7 +755,7 @@ void MainMode::setupLighting(const Game *game, bool isNight) {
   const Coord3d moonPosition(0., 0., 2.);
   GLfloat black[] = {0.0, 0.0, 0.0, 1.0};
   if (isNight && game && game->player1) {
-    GLfloat lightDiffuse2[3] = {0.9, 0.9, 0.9};
+    GLfloat lightDiffuse2[3] = {0.5, 0.5, 0.5};
     GLfloat lightPosition2[3] = {(GLfloat)(game->player1->position[0] + moonPosition[0]),
                                  (GLfloat)(game->player1->position[1] + moonPosition[1]),
                                  (GLfloat)(game->player1->position[2] + moonPosition[2])};
