@@ -72,6 +72,8 @@ char *story;
 void MenuMode::display() {
   char str[256];
 
+  /* This text used to be drawn on the main screen, possibly as a single long text
+   * scrolling leftwards */
   story =
       _("Trackballs is a game similar to the classical game Marble Madness from the 80's. \
 By steering a marble ball through a labyrinth filled with sharp spikes, \
@@ -96,8 +98,8 @@ You steer the ball using the mouse and by pressing >spacebar< you can jump a sho
   }
 
   /* Draw header */
-  draw2DRectangle(screenWidth / 2 - 512 / 2, 20, 512, 128, 0., 0., 1., 1., 1., 1., 1., 1.,
-                  textures[loadTexture("header.png")]);
+  draw2DRectangle(screenWidth / 2 - 512 / 2, 20, 512, 128, 0., 0., 1., 1.,
+                  Color(1., 1., 1., 1.), textures[loadTexture("header.png")]);
 
   clearSelectionAreas();
   int fontSize = computeMenuSize();
@@ -120,7 +122,8 @@ You steer the ball using the mouse and by pressing >spacebar< you can jump a sho
   */
 
   sprintf(str, "v%s", VERSION);
-  drawRightSimpleText(str, screenWidth - 15, screenHeight - 15, 10, 0.50, 0.50, 0.50, 0.75);
+  drawRightSimpleText(str, screenWidth - 15, screenHeight - 15, 10,
+                      Color(SRGBColor(0.50, 0.50, 0.50, 0.75)));
 
   drawMousePointer();
   displayFrameRate();
@@ -163,12 +166,6 @@ void MenuMode::tick(Real td) {
   tickMouse(td);
   const Uint8 *keystate = SDL_GetKeyboardState(NULL);
   if (keystate[SDL_SCANCODE_SPACE] || keystate[SDL_SCANCODE_RETURN]) doSelection();
-
-  /* This controls the scrolling text */
-  offset += 150.0 * td;
-  static int stwidth = -1;
-  if (stwidth < 0) { stwidth = getTextWidth(story, 24); }
-  if (offset > stwidth + screenWidth) offset = 0;
 }
 void MenuMode::mouseDown(int /*mouseState*/, int /*x*/, int /*y*/) { doSelection(); }
 
@@ -285,5 +282,5 @@ void MenuMode::drawSlide(int slide, double time, double alpha) {
   }
 
   draw2DRectangle(0, 0, screenWidth, screenHeight, viewOffsetX, viewOffsetY, viewWidth,
-                  viewHeight, 1., 1., 1., alpha, slides[slide]);
+                  viewHeight, Color(1., 1., 1., alpha), slides[slide]);
 }
