@@ -151,6 +151,8 @@ static SDL_GLContext createWindow() {
   SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 3);
   SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
 
+  SDL_GL_SetAttribute(SDL_GL_FRAMEBUFFER_SRGB_CAPABLE, 1);
+
   /* Uncomment to apply basic antialiasing.
    * SDL can't recover from failure to obtain this if asked for */
   //    SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
@@ -300,7 +302,7 @@ static void *mainLoop(void *data) {
     glClear(GL_COLOR_BUFFER_BIT);
     Enter2DMode();
     draw2DRectangle(0, 0, screenWidth, screenHeight, texcoord[0], texcoord[1], texcoord[2],
-                    texcoord[3], 1., 1., 1., 1., splashTexture);
+                    texcoord[3], Color(1., 1., 1., 1.), splashTexture);
     Leave2DMode();
     SDL_GL_SwapWindow(window);
   }
@@ -331,7 +333,7 @@ static void *mainLoop(void *data) {
     glClear(GL_COLOR_BUFFER_BIT);
     Enter2DMode();
     draw2DRectangle(0, 0, screenWidth, screenHeight, texcoord[0], texcoord[1], texcoord[2],
-                    texcoord[3], 1., 1., 1., 1., splashTexture);
+                    texcoord[3], Color(1., 1., 1., 1.), splashTexture);
     Leave2DMode();
     SDL_GL_SwapWindow(window);
     timeNow = getMonotonicTime();
@@ -347,7 +349,6 @@ static void *mainLoop(void *data) {
   /* Initialize random number generator */
   int seed = getMonotonicTime().tv_nsec;
   srand(seed);
-  int keyUpReceived = 1;
 
   double loop_time = 0.;
   Uint32 zero_sdl_tick = SDL_GetTicks();
@@ -402,9 +403,6 @@ static void *mainLoop(void *data) {
           GameMode::current->mouseDown(event.button.button, event.button.x, event.button.y);
         break;
       case SDL_KEYUP:
-        /* Prevent repeated keys */
-        keyUpReceived = 1;
-
         /* Use Caps lock key to determine if mouse should be hidden+grabbed */
         if (event.key.keysym.sym == SDLK_CAPSLOCK) {
           if (SDL_GetModState() & KMOD_CAPS) {
@@ -444,14 +442,11 @@ static void *mainLoop(void *data) {
             EditMode::editMode->askQuit();
           } else if ((GameMode::current && GameMode::current == MenuMode::menuMode))
             GameMode::activate(NULL);
-          else {
-            GameMode::activate(MenuMode::init());
-          }
+          else { GameMode::activate(MenuMode::init()); }
 
         } else if (GameMode::current) {
           /* Prevent repeated keys */
-          if (!keyUpReceived) break;
-          keyUpReceived = 0;
+          if (event.key.repeat) { break; }
 
           GameMode::current->key(event.key.keysym.sym);
         }
