@@ -44,6 +44,7 @@ Sign::Sign(Game &g, const char *string, Real l, Real s, Real r, const Coord3d &p
   }
 
   textimg = 0;
+  textureChanged = true;
   mkTexture(string);
 }
 
@@ -60,46 +61,47 @@ void Sign::mkTexture(const char *string) {
   width = text->w;
   height = text->h;
   SDL_FreeSurface(text);
+  textureChanged = true;
 
-  /* Note oversided bounding box since the sign rotates */
-  boundingBox[0][0] = -std::sqrt(2) * SIGN_SCALE * scale * width;
-  boundingBox[1][0] = +std::sqrt(2) * SIGN_SCALE * scale * width;
-  boundingBox[0][1] = -std::sqrt(2) * SIGN_SCALE * scale * width;
-  boundingBox[1][1] = +std::sqrt(2) * SIGN_SCALE * scale * width;
+  /* This bounding box contains the sign as it rotates  */
+  boundingBox[0][0] = -SIGN_SCALE * scale * width;
+  boundingBox[1][0] = +SIGN_SCALE * scale * width;
+  boundingBox[0][1] = -SIGN_SCALE * scale * width;
+  boundingBox[1][1] = +SIGN_SCALE * scale * width;
   boundingBox[0][2] = -SIGN_SCALE * scale * height;
   boundingBox[1][2] = SIGN_SCALE * scale * height;
 }
 
-void Sign::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
-                           const GLuint *vaolist, bool /*mustUpdate*/) const {
+void Sign::updateBuffers(const GLuint *idxbufs, const GLuint *databufs, const GLuint *vaolist,
+                         bool firstCall) {
+  if (!(firstCall || textureChanged)) { return; }
+  textureChanged = false;
+
   GLfloat flat[3] = {0.f, 0.f, 0.f};
 
   GLfloat data[8 * 8];
   char *pos = (char *)data;
-  GLfloat dx = std::cos(M_PI * (tot_rot - 45.) / 180.) * SIGN_SCALE * scale * width;
-  GLfloat dy = std::sin(M_PI * (tot_rot - 45.) / 180.) * SIGN_SCALE * scale * width;
-  GLfloat dz = SIGN_SCALE * scale * height;
+  GLfloat dx = SIGN_SCALE * width;
+  GLfloat dz = SIGN_SCALE * height;
 
   Color color = primaryColor.toOpaque();
-  color.v[3] = std::min(1.0, life) * 65535;
+  color.v[3] = std::min(1.0, life) * 1.f;
 
-  pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] + dz,
-                          texcoord[0], texcoord[1], color, flat);
-  pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] - dz,
-                          texcoord[0], texcoord[1] + texcoord[3], color, flat);
-  pos += packObjectVertex(pos, position[0] - dx, position[1] - dy, position[2] + dz,
-                          texcoord[0] + texcoord[2], texcoord[1], color, flat);
-  pos += packObjectVertex(pos, position[0] - dx, position[1] - dy, position[2] - dz,
-                          texcoord[0] + texcoord[2], texcoord[1] + texcoord[3], color, flat);
+  pos += packObjectVertex(pos, dx, 0., dz, texcoord[0], texcoord[1], color, flat);
+  pos +=
+      packObjectVertex(pos, dx, 0., -dz, texcoord[0], texcoord[1] + texcoord[3], color, flat);
+  pos +=
+      packObjectVertex(pos, -dx, 0., +dz, texcoord[0] + texcoord[2], texcoord[1], color, flat);
+  pos += packObjectVertex(pos, -dx, 0., -dz, texcoord[0] + texcoord[2],
+                          texcoord[1] + texcoord[3], color, flat);
 
-  pos += packObjectVertex(pos, position[0] - dx, position[1] - dy, position[2] + dz,
-                          texcoord[0], texcoord[1], color, flat);
-  pos += packObjectVertex(pos, position[0] - dx, position[1] - dy, position[2] - dz,
-                          texcoord[0], texcoord[1] + texcoord[3], color, flat);
-  pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] + dz,
-                          texcoord[0] + texcoord[2], texcoord[1], color, flat);
-  pos += packObjectVertex(pos, position[0] + dx, position[1] + dy, position[2] - dz,
-                          texcoord[0] + texcoord[2], texcoord[1] + texcoord[3], color, flat);
+  pos += packObjectVertex(pos, -dx, 0., +dz, texcoord[0], texcoord[1], color, flat);
+  pos +=
+      packObjectVertex(pos, -dx, 0., -dz, texcoord[0], texcoord[1] + texcoord[3], color, flat);
+  pos +=
+      packObjectVertex(pos, +dx, 0., +dz, texcoord[0] + texcoord[2], texcoord[1], color, flat);
+  pos += packObjectVertex(pos, +dx, 0., -dz, texcoord[0] + texcoord[2],
+                          texcoord[1] + texcoord[3], color, flat);
 
   glBindVertexArray(vaolist[0]);
   glBindBuffer(GL_ARRAY_BUFFER, databufs[0]);
@@ -115,20 +117,23 @@ void Sign::drawBuffers1(const GLuint * /*vaolist*/) const {}
 void Sign::drawBuffers2(const GLuint *vaolist) const {
   if (activeView.calculating_shadows) return;
 
-  // Keep the depth function on but trivial so as to record depth values
   glEnable(GL_BLEND);
   glEnable(GL_CULL_FACE);
-  glDepthFunc(GL_ALWAYS);
 
   // Transfer data
-  setActiveProgramAndUniforms(Shader_Object);
-  setObjectUniforms(Color(0., 0., 0., 1.), 0., Lighting_None);
+  Matrix4d transform;
+  affineMatrix(transform, identity3, position);
+  rotateZ(M_PI * (tot_rot + 45.) / 180., transform);
+  for (int i = 0; i < 3; i++) {
+    for (int j = 0; j < 3; j++) { transform[i][j] *= scale; }
+  }
+
+  const UniformLocations *uloc = setActiveProgramAndUniforms(Shader_Object);
+  setObjectUniforms(uloc, transform, Color(0., 0., 0., 1.), 0., Lighting_None);
   glBindTexture(GL_TEXTURE_2D, textimg);
 
   glBindVertexArray(vaolist[0]);
   glDrawElements(GL_TRIANGLES, 12, GL_UNSIGNED_SHORT, (void *)0);
-
-  glDepthFunc(GL_LEQUAL);
 }
 
 void Sign::tick(Real t) {
